using Microsoft.Extensions.Logging;
using Piwigo.Client.Images;

namespace Piwigo.Client.Cli;

internal static class UploadHandler
{
    public static async Task<int> UploadAndReturnExitCode(UploadOptions options, ILoggerFactory loggerFactory)
    {
        var logger = loggerFactory.CreateLogger(nameof(UploadHandler));

        try
        {
            var imageToUpload = new FileInfo(options.ImagePath);
            var md5Sum = await imageToUpload.CalculateMd5SumAsync();

            logger.LogInformation(EventIds.Upload, "Uploading {ImagePath}", options.ImagePath);
            var client = PiwigoClient.CreateClient(options.ServerUrl, options.UserName, options.Password, loggerFactory);
            await client.Session.LoginAsync();

            var imageUpload = new ImageUpload(md5Sum)
            {
                Albums = options.AlbumId.HasValue ? new List<(int AlbumId, int? Rank)> { new(options.AlbumId.Value, null) } : null
            };

            var result = await client.UploadImageAsync(imageToUpload, imageUpload);
            logger.LogInformation(EventIds.ActionSucceeded, "Uploaded {ImagePath} with new image id {ImageId}", options.ImagePath, result.ImageId);

            return EventIds.ActionSucceeded.Id;
        }
        catch (PiwigoException ex)
        {
            logger.LogCritical(EventIds.PiwigoError, ex, "{Message}", ex.Message);
            return EventIds.PiwigoError.Id;
        }
        catch (Exception ex)
        {
            logger.LogCritical(EventIds.SystemError, ex, "{Message}", ex.Message);
            return EventIds.SystemError.Id;
        }
    }
}