using Microsoft.Extensions.Logging.Abstractions;
using Piwigo.Client.Tags;

namespace Piwigo.Client.Tests;

[TestFixture]
public class TagApiTests : ApiTestsBase
{
    private ITagApi _tagApi = null!;
    private const string ApiToken = "37ff55f201cf54eadf11f734a74f1d0e";

    protected override void OnSetUp()
    {
        base.OnSetUp();
        _tagApi = new TagApi(Context, new NullLogger<TagApi>());
    }

    [Test]
    public async Task AddAsync_should_add_tag_and_return_correct_result()
    {
        SetJsonResult(@"{""stat"":""ok"",""result"":{""info"":""Keyword \""test3\"" has been added"",""id"":3,""name"":""test3"",""url_name"":""test3""}}");

        var response = await _tagApi.AddAsync("test3");

        CorrectMethodShouldGetCalled("pwg.tags.add");
        CorrectParamShouldGetSent("name", "test3");

        await Verify(response);
    }

    [Test]
    public async Task GetAdminListAsync_should_return_all_tags()
    {
        await SetJsonResultFromFileAsync("TagApi.GetAdminList.json");

        var response = await _tagApi.GetAdminListAsync();

        CorrectMethodShouldGetCalled("pwg.tags.getAdminList");

        await Verify(response);
    }

    [Test]
    public async Task GetListAsync_should_return_all_tags()
    {
        await SetJsonResultFromFileAsync("TagApi.GetList.json");

        var response = await _tagApi.GetListAsync();

        CorrectMethodShouldGetCalled("pwg.tags.getList");

        await Verify(response);
    }

    [Test]
    public async Task DeleteAsync_should_pass_correct_parameters()
    {
        SetOkResult();

        await _tagApi.DeleteAsync(123, ApiToken);

        CorrectMethodShouldGetCalled("pwg.tags.delete");
        CorrectParamShouldGetSent("tag_id", "123");
        CorrectParamShouldGetSent("pwg_token", ApiToken);
    }

    [Test]
    public async Task DuplicateAsync_should_pass_correct_parameters()
    {
        SetJsonResult(@"{""stat"":""ok"",""result"":{""id"":4,""name"":""dup2"",""url_name"":""dup2"",""count"":1}}");

        var response = await _tagApi.DuplicateAsync(123, "newTag", ApiToken);

        CorrectMethodShouldGetCalled("pwg.tags.duplicate");
        CorrectParamShouldGetSent("tag_id", "123");
        CorrectParamShouldGetSent("copy_name", "newTag");
        CorrectParamShouldGetSent("pwg_token", ApiToken);

        await Verify(response);
    }

    [Test]
    public async Task MergeAsync_should_pass_correct_parameters()
    {
        await SetJsonResultFromFileAsync("TagApi.Merge.json");

        var response = await _tagApi.MergeAsync(1, 4, ApiToken);

        CorrectMethodShouldGetCalled("pwg.tags.merge");
        CorrectParamShouldGetSent("destination_tag_id", "1");
        CorrectParamShouldGetSent("merge_tag_id", "4");
        CorrectParamShouldGetSent("pwg_token", ApiToken);

        await Verify(response);
    }

    [Test]
    public async Task RenameAsync_should_pass_correct_parameters()
    {
        SetJsonResult(@"{""stat"":""ok"",""result"":{""id"":1,""name"":""tag1new"",""url_name"":""tag1new""}}");

        await _tagApi.RenameAsync(1, "tag1new", ApiToken);

        CorrectMethodShouldGetCalled("pwg.tags.rename");
        CorrectParamShouldGetSent("tag_id", "1");
        CorrectParamShouldGetSent("new_name", "tag1new");
        CorrectParamShouldGetSent("pwg_token", ApiToken);
    }
}