namespace Piwigo.Client.Session;

public interface ISessionApi
{
    /// <summary>
    ///     Logs into the configured piwigo server using the configured credentials.
    ///     <see cref="IPiwigoConfiguration" />
    /// </summary>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    Task LoginAsync(CancellationToken cancellationToken = default);

    /// <summary>
    ///     Closes current session and logs out of the configured piwigo server.
    ///     <see cref="IPiwigoConfiguration" />
    /// </summary>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    Task LogoutAsync(CancellationToken cancellationToken = default);

    /// <summary>
    ///     Gets the status of the current session including some useful information about the server.
    /// </summary>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    Task<SessionStatus> GetStatusAsync(CancellationToken cancellationToken = default);

    /// <summary>
    ///     Gets the upload chunk size that the server specifies for image upload.
    ///     The chunk size is only get once and cleared on logout.
    /// </summary>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    ValueTask<int> GetUploadChunkSizeAsync(CancellationToken cancellationToken = default);

    /// <summary>
    ///     Gets the file types supported by the piwigo instance.
    ///     The chunk size is only get once and cleared on logout.
    /// </summary>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    ValueTask<IReadOnlySet<string>> GetSupportedFileTypes(CancellationToken cancellationToken = default);
}